/*
 * @author Stefan Hepp
 * @date 16.08.2005
 * @version $Id: GenticsHistoryModule.java,v 1.11 2006-01-23 17:10:06 norbert Exp $
 */

package com.gentics.portalnode.sdk;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.PortletURL;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import com.gentics.api.portalnode.event.ActionEvent;
import com.gentics.api.portalnode.portlet.AbstractGenticsPortlet;
import com.gentics.api.portalnode.templateengine.PrivateKeyException;
import com.gentics.api.portalnode.templateengine.TemplateNotFoundException;
import com.gentics.api.portalnode.templateengine.TemplateProcessor;

/**
 * A simple history module which can hold a history of strings, new history entries can be added by a
 * module parameter or by a session attribute.
 */
public class GenticsHistoryModule extends AbstractGenticsPortlet {

    // names of internal session- and parameternames
    private static final String SESSION_HISTORY = "com.gentics.HistoryModule.history";
    private static final String DATA_PARAMETER = "data";

    // name of public session parameter for communication
    public static final String SESSION_DATA = "com.gentics.HistoryModule.session";

    public GenticsHistoryModule(String moduleId) throws PortletException {
        super(moduleId);
    }

    /**
     * react on changes of the module's data parameter.
     */

    protected void onParameterChanged(String parameterName, ActionEvent e) {
        super.onParameterChanged(parameterName, e);

        if (DATA_PARAMETER.equals(parameterName)) {

            // get session
            PortletSession session = getPortletRequest().getPortletSession();

            // get data-parameter and check if it is set
            String data = getGenticsPortletContext().getStringModuleParameter(DATA_PARAMETER);

            if (data != null && !"".equals(data)) {

                // get the local history from the session, add the new data to the history
                List history = getHistory(session);
                history.add(data);

            }

            getGenticsPortletContext().setModuleParameter(DATA_PARAMETER, "");
        }

    }

    public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {

        // if a clear action has been triggered, clear the history in the session
        if ("clear".equals(request.getParameter("action"))) {
            List history = (List) request.getPortletSession().getAttribute(SESSION_HISTORY);
            if (history != null) {
                history.clear();
            }
        }
    }

    protected void doView(RenderRequest renderRequest, RenderResponse renderResponse) throws PortletException, IOException {
        PortletSession session = renderRequest.getPortletSession();

        // get the history from the session
        List history = getHistory(session);

        // check if new data has been stored into the session
        Object data = session.getAttribute(SESSION_DATA, PortletSession.APPLICATION_SCOPE);
        if (data != null && data instanceof Collection) {

            // get all elements from the session-attribute and add them to the history
            for (Iterator iterator = ((Collection) data).iterator(); iterator.hasNext();) {
                Object o = iterator.next();
                if (o != null) {
                    history.add(o.toString());
                }
            }

            // clear the data-list in the session
            ((Collection) data).clear();
        }

        // render the output using the templateprocessor
        TemplateProcessor templateProcessor = getTemplateProcessor(renderRequest, renderResponse);

        try {

            // create an actionurl which clears the history
            PortletURL actionURL = renderResponse.createActionURL();
            actionURL.setParameter("action", "clear");

            // get the templateprocessor and add the variables to it
            templateProcessor.put("history", history);
            templateProcessor.put("clearlink", actionURL);

            // render the main-template
            renderResponse.getWriter().print(templateProcessor.getOutput("main", this));

        } catch (PrivateKeyException e) {
            logger.error(e);
        } catch (TemplateNotFoundException e) {
            logger.error(e);
        } finally {
            // return the template processor
            returnTemplateProcessor(templateProcessor);
        }
    }

    /**
     * get the history list from the session. Create a new history if neccessary.
     *
     * @param session the session where the history should be stored.
     * @return the current history-object from the session as a (sorted) list of strings.
     */
    private List getHistory(PortletSession session) {

        // get the history from the session
        List history = (List) session.getAttribute(SESSION_HISTORY);

        // if no history has been created yet, create one and add it to the session.
        if (history == null) {
            history = new ArrayList(10);
            session.setAttribute(SESSION_HISTORY, history);
        }

        return history;
    }

}
